"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const chai_1 = require("chai");
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.HASURA_SSL,
    secret: process.env.HASURA_SECRET,
});
const root = new client_2.DeepClient({ apolloClient });
let adminToken;
let admin;
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    const { linkId, token } = yield root.jwt({ linkId: yield root.id('@deep-foundation/core', 'system', 'admin') });
    adminToken = token;
    admin = new client_2.DeepClient({ deep: root, token: adminToken, linkId });
}));
let deep = root;
describe('permissions', () => {
    describe('select', () => {
        it(`user contain range`, () => __awaiter(void 0, void 0, void 0, function* () {
            const a1 = yield deep.guest({});
            const a2 = yield deep.guest({});
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const d2 = new client_2.DeepClient(Object.assign({ deep }, a2));
            const { data: [{ id }] } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                in: { data: {
                        type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                        from_id: a1.linkId,
                    } }
            });
            const n1 = yield d1.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const n2 = yield d2.select({ id });
            chai_1.assert.lengthOf(n2 === null || n2 === void 0 ? void 0 : n2.data, 0);
            const n3 = yield admin.select({ id });
            chai_1.assert.lengthOf(n3 === null || n3 === void 0 ? void 0 : n3.data, 1);
        }));
        it(`rule select 1 depth`, () => __awaiter(void 0, void 0, void 0, function* () {
            const a1 = yield deep.guest({});
            const a2 = yield deep.guest({});
            const a3 = yield deep.guest({});
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const d2 = new client_2.DeepClient(Object.assign({ deep }, a2));
            const d3 = new client_2.DeepClient(Object.assign({ deep }, a3));
            const { data: [{ id }] } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                in: { data: {
                        type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                        from_id: a1.linkId,
                    } }
            });
            yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: a2.linkId,
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: id
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core', 'Select'),
                                        } }
                                } }
                        },
                    ] },
            });
            const n1 = yield d1.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const n2 = yield d2.select({ id });
            chai_1.assert.lengthOf(n2 === null || n2 === void 0 ? void 0 : n2.data, 1);
            const n3 = yield d3.select({ id });
            chai_1.assert.lengthOf(n3 === null || n3 === void 0 ? void 0 : n3.data, 0);
            const n4 = yield admin.select({ id });
            chai_1.assert.lengthOf(n4 === null || n4 === void 0 ? void 0 : n4.data, 1);
        }));
        it(`rule select 2 depth`, () => __awaiter(void 0, void 0, void 0, function* () {
            const a1 = yield deep.guest({});
            const a2 = yield deep.guest({});
            const a3 = yield deep.guest({});
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const d2 = new client_2.DeepClient(Object.assign({ deep }, a2));
            const d3 = new client_2.DeepClient(Object.assign({ deep }, a3));
            const { data: [{ id }] } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                in: { data: {
                        type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                        from: { data: {
                                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                                in: { data: {
                                        type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                                        from_id: a1.linkId,
                                    } },
                            } },
                    } }
            });
            yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: a2.linkId,
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: id
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core', 'Select'),
                                        } }
                                } }
                        },
                    ] },
            });
            const n1 = yield d1.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const n2 = yield d2.select({ id });
            chai_1.assert.lengthOf(n2 === null || n2 === void 0 ? void 0 : n2.data, 1);
            const n3 = yield d3.select({ id });
            chai_1.assert.lengthOf(n3 === null || n3 === void 0 ? void 0 : n3.data, 0);
            const n4 = yield admin.select({ id });
            chai_1.assert.lengthOf(n4 === null || n4 === void 0 ? void 0 : n4.data, 1);
        }));
    });
    describe('insert', () => {
        it(`root can insert`, () => __awaiter(void 0, void 0, void 0, function* () {
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
            });
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
        }));
        it(`guest cant insert by default`, () => __awaiter(void 0, void 0, void 0, function* () {
            const a1 = yield deep.guest({});
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            let trowed = false;
            let _id;
            try {
                const { data: [{ id }], error } = yield d1.insert({
                    type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                });
                _id = id;
            }
            catch (error) {
                trowed = true;
            }
            chai_1.assert.equal(trowed, true);
        }));
        it(`insert permission can be gived to guest`, () => __awaiter(void 0, void 0, void 0, function* () {
            const a1 = yield deep.guest({});
            yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: a1.linkId,
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core'),
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core', 'Insert'),
                                        } }
                                } }
                        },
                    ] },
            });
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const { data: [{ id }], error } = yield d1.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
            });
        }));
    });
    describe('update', () => {
        it(`root can update string value`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c;
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                string: { data: { value: 'abc' } },
            });
            yield deep.update({ link_id: id }, {
                value: 'def',
            }, { table: 'strings' });
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            chai_1.assert.equal((_c = (_b = (_a = n1 === null || n1 === void 0 ? void 0 : n1.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.value, 'def');
        }));
        it(`guest cant update string value`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _d, _e, _f;
            const a1 = yield deep.guest({});
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                string: { data: { value: 'abc' } },
            });
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const { data: updated } = yield d1.update({ link_id: id }, {
                value: 'def',
            }, { table: 'strings' });
            chai_1.assert.lengthOf(updated, 0);
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            chai_1.assert.equal((_f = (_e = (_d = n1 === null || n1 === void 0 ? void 0 : n1.data) === null || _d === void 0 ? void 0 : _d[0]) === null || _e === void 0 ? void 0 : _e.value) === null || _f === void 0 ? void 0 : _f.value, 'abc');
        }));
        it(`update permission can be gived to guest`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _g, _h, _j;
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                string: { data: { value: 'abc' } },
            });
            const a1 = yield deep.guest({});
            yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: a1.linkId,
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core'),
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core', 'Update'),
                                        } }
                                } }
                        },
                    ] },
            });
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const { data: updated } = yield d1.update({ link_id: id }, {
                value: 'def',
            }, { table: 'strings' });
            chai_1.assert.lengthOf(updated, 1);
            const n1 = yield deep.select({ id });
            console.log(updated, n1 === null || n1 === void 0 ? void 0 : n1.data);
            chai_1.assert.equal((_j = (_h = (_g = n1 === null || n1 === void 0 ? void 0 : n1.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.value) === null || _j === void 0 ? void 0 : _j.value, 'def');
        }));
    });
    describe('delete', () => {
        it(`root can delete`, () => __awaiter(void 0, void 0, void 0, function* () {
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
            });
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const { data: deleted } = yield deep.delete(id);
            chai_1.assert.lengthOf(deleted, 1);
            const n2 = yield deep.select({ id });
            chai_1.assert.lengthOf(n2 === null || n2 === void 0 ? void 0 : n2.data, 0);
        }));
        it(`guest cant delete by default`, () => __awaiter(void 0, void 0, void 0, function* () {
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
            });
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const a1 = yield deep.guest({});
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            const { data: deleted } = yield d1.delete(id);
            chai_1.assert.lengthOf(deleted, 0);
            const n2 = yield deep.select({ id });
        }));
        it(`delete permission can be gived to guest`, () => __awaiter(void 0, void 0, void 0, function* () {
            const { data: [{ id }], error } = yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
            });
            const a1 = yield deep.guest({});
            yield deep.insert({
                type_id: yield deep.id('@deep-foundation/core', 'Rule'),
                out: { data: [
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleSubject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: a1.linkId,
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleObject'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core'),
                                        } }
                                } }
                        },
                        {
                            type_id: yield deep.id('@deep-foundation/core', 'RuleAction'),
                            to: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'Selector'),
                                    out: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Selection'),
                                            to_id: yield deep.id('@deep-foundation/core', 'Delete'),
                                        } }
                                } }
                        },
                    ] },
            });
            const n1 = yield deep.select({ id });
            chai_1.assert.lengthOf(n1 === null || n1 === void 0 ? void 0 : n1.data, 1);
            const d1 = new client_2.DeepClient(Object.assign({ deep }, a1));
            yield d1.delete(id);
            const n2 = yield deep.select({ id });
            chai_1.assert.lengthOf(n2 === null || n2 === void 0 ? void 0 : n2.data, 0);
        }));
    });
});
//# sourceMappingURL=permissions.js.map